module Statement(T, parse, toString, fromString, exec) where
import Prelude hiding (return, fail, read)
import Parser hiding (T)
import qualified Dictionary
import qualified Expr
type T = Statement
data Statement =
    Assignment String Expr.T |
    Skip | 
    Begin [Statement] |
    If Expr.T Statement Statement |
    While Expr.T Statement |
    Read String |
    Write Expr.T
    deriving Show

assignment = word #- accept ":=" # Expr.parse #- require ";" >-> buildAss
	where buildAss (v, e) = Assignment v e

skip = accept "skip" # require ";" >-> buildSkip
	where buildSkip _ = Skip

begin = accept "begin" -# iter parse #- require "end" >-> buildBegin
	where buildBegin statements = Begin statements

if_parse = accept "if" -# Expr.parse # require "then" -# parse # require "else" -# parse >-> buildIf
	where buildIf ((e, s1), s2) = If e s1 s2

while = accept "while" -# Expr.parse # require "do" -# parse >-> buildWhile
	where buildWhile (e, s) = While e s

read = accept "read" -# word #- require ";" >-> buildRead
	where buildRead v = Read v

write = accept "write" -# Expr.parse #- require ";" >-> buildWrite
	where buildWrite e = Write e




exec :: [T] -> Dictionary.T String Integer -> [Integer] -> [Integer]
-- End of program
exec [] _ _ = []

-- Assignment
exec (Assignment var expr: stmts) dict input = 
	exec stmts (Dictionary.insert (var, Expr.value expr dict) dict) input

-- Skip
exec (Skip: stmts) dict input =
	exec stmts dict input

-- Begin
exec (Begin list: stmts) dict input =
	exec (list++stmts) dict input

-- If
exec (If cond thenStmts elseStmts: stmts) dict input = 
    if (Expr.value cond dict)>0 
    then exec (thenStmts: stmts) dict input
    else exec (elseStmts: stmts) dict input

-- While
exec (While cond doStmt: stmts) dict input = 
	if (Expr.value cond dict) > 0
	then exec (doStmt: While cond doStmt: stmts) dict input
	else exec stmts dict input

-- Read
exec (Read var: stmts) dict (first:input) =
	exec stmts (Dictionary.insert (var,first) dict) input

-- Write
exec (Write expr: stmts) dict input =
	Expr.value expr dict :
	exec stmts dict input


instance Parse Statement where
	parse = assignment ! skip ! begin ! if_parse ! while ! read ! write
	
	-- Assignment
	toString (Assignment var expr) = 
		var ++ " := " ++ Expr.toString expr ++ "\n"
		
	-- Skip
	toString (Skip) = 
		 "skip;\n"

	-- Begin
	toString (Begin list) = 
		"begin\n" ++ concat (map toString list) ++ "end\n"

	-- If
	toString (If cond thenStmts elseStmts) =
		"if " ++ Expr.toString cond ++ " then\n" ++ toString thenStmts ++ "else\n" ++ toString elseStmts
	
	-- While
	toString (While cond doStmt) =
		"while " ++ Expr.toString cond ++ " do\n" ++ toString doStmt
	
	-- Read
	toString (Read var) =
		"read " ++ var ++ ";\n"
	
	-- Write
	toString (Write expr) = 
		"write " ++ Expr.toString expr ++ ";\n";
